@@ -38,10 +38,7 @@ import java.security.DigestInputStream;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
 import java.security.ProtectionDomain;
-import java.util.ArrayList;
-import java.util.Enumeration;
-import java.util.List;
-import java.util.Properties;
+import java.util.*;
 
 /**
  * <b>Internal only - Do not use this class.</b> This class loads a native
@@ -93,9 +90,19 @@ public class SnappyLoader
     public static final String KEY_SNAPPY_USE_SYSTEMLIB        = "org.xerial.snappy.use.systemlib";
     public static final String KEY_SNAPPY_DISABLE_BUNDLED_LIBS = "org.xerial.snappy.disable.bundled.libs"; // Depreciated, but preserved for backward compatibility
 
-    private static boolean     isLoaded                        = false;
-    private static Object      api                             = null;
-
+    private static volatile boolean     isLoaded                        = false;
+    private static volatile Object api                             = null;
+    
+    /**
+     * Set the api instance.
+     * 
+     * @param nativeCode
+     */
+    static synchronized void setApi(Object nativeCode)
+    {
+    	api = nativeCode;
+    }
+    
     /**
      * load system properties when configuration file of the name
      * {@link #SNAPPY_SYSTEM_PROPERTIES_FILE} is found
@@ -206,8 +213,8 @@ public class SnappyLoader
      * 
      * @return
      */
-    static synchronized Object load() {
-
+    static synchronized Object load() 
+    {
         if (api != null)
             return api;
 
@@ -222,7 +229,7 @@ public class SnappyLoader
             isLoaded = true;
             // Look up SnappyNative, injected to the root classloder, using reflection in order to avoid the initialization of SnappyNative class in this context class loader.
             Object nativeCode = Class.forName("org.xerial.snappy.SnappyNative").newInstance();
-            api = nativeCode;
+            setApi(nativeCode);
         }
         catch (Exception e) {
             e.printStackTrace();
@@ -356,8 +363,10 @@ public class SnappyLoader
      */
     private static File extractLibraryFile(String libFolderForCurrentOS, String libraryFileName, String targetFolder) {
         String nativeLibraryFilePath = libFolderForCurrentOS + "/" + libraryFileName;
-        final String prefix = "snappy-" + getVersion() + "-";
-        String extractedLibFileName = prefix + libraryFileName;
+
+        // Attach UUID to the native library file to ensure multiple class loaders can read the libsnappy-java multiple times.
+        String uuid = UUID.randomUUID().toString();
+        String extractedLibFileName = String.format("snappy-%s-%s-%s", getVersion(), uuid, libraryFileName);
         File extractedLibFile = new File(targetFolder, extractedLibFileName);
 
         try {
@@ -411,13 +420,10 @@ public class SnappyLoader
     static File findNativeLibrary() {
 
         boolean useSystemLib = Boolean.parseBoolean(System.getProperty(KEY_SNAPPY_USE_SYSTEMLIB, "false"));
-        if (useSystemLib)
-            return null;
-
         boolean disabledBundledLibs = Boolean
                 .parseBoolean(System.getProperty(KEY_SNAPPY_DISABLE_BUNDLED_LIBS, "false"));
-        if (disabledBundledLibs)
-            return null;
+        if (useSystemLib || disabledBundledLibs)
+            return null; // Use a pre-installed libsnappyjava
 
         // Try to load the library in org.xerial.snappy.lib.path  */
         String snappyNativeLibraryPath = System.getProperty(KEY_SNAPPY_LIB_PATH);
@@ -433,23 +439,43 @@ public class SnappyLoader
                 return nativeLib;
         }
 
-        {
-            // Load an OS-dependent native library inside a jar file
-            snappyNativeLibraryPath = "/org/xerial/snappy/native/" + OSInfo.getNativeLibFolderPathForCurrentOS();
 
-            if (SnappyLoader.class.getResource(snappyNativeLibraryPath + "/" + snappyNativeLibraryName) != null) {
-                // Temporary library folder. Use the value of org.xerial.snappy.tempdir or java.io.tmpdir
-                String tempFolder = new File(System.getProperty(KEY_SNAPPY_TEMPDIR,
-                        System.getProperty("java.io.tmpdir"))).getAbsolutePath();
-
-                // Extract and load a native library inside the jar file
-                return extractLibraryFile(snappyNativeLibraryPath, snappyNativeLibraryName, tempFolder);
+        // Load an OS-dependent native library inside a jar file
+        snappyNativeLibraryPath = "/org/xerial/snappy/native/" + OSInfo.getNativeLibFolderPathForCurrentOS();
+        boolean hasNativeLib = hasResource(snappyNativeLibraryPath + "/" + snappyNativeLibraryName);
+        if(!hasNativeLib) {
+            if(OSInfo.getOSName().equals("Mac")) {
+                // Fix for openjdk7 for Mac
+                String altName = "libsnappyjava.jnilib";
+                if(hasResource(snappyNativeLibraryPath + "/" + altName)) {
+                    snappyNativeLibraryName = altName;
+                    hasNativeLib = true;
+                }
             }
         }
 
-        return null; // Use a pre-installed libsnappyjava
+        if(!hasNativeLib) {
+            String errorMessage = String.format("no native library is found for os.name=%s and os.arch=%s", OSInfo.getOSName(), OSInfo.getArchName());
+            throw new SnappyError(SnappyErrorCode.FAILED_TO_LOAD_NATIVE_LIBRARY, errorMessage);
+        }
+
+        // Temporary library folder. Use the value of org.xerial.snappy.tempdir or java.io.tmpdir
+        String tempFolder = new File(System.getProperty(KEY_SNAPPY_TEMPDIR,
+                System.getProperty("java.io.tmpdir"))).getAbsolutePath();
+
+        // Extract and load a native library inside the jar file
+        return extractLibraryFile(snappyNativeLibraryPath, snappyNativeLibraryName, tempFolder);
     }
 
+
+    private static boolean hasResource(String path) {
+        return SnappyLoader.class.getResource(path) != null;
+    }
+
+
+
+
+
     /**
      * Get the snappy-java version by reading pom.properties embedded in jar.
      * This version data is used as a suffix of a dll file extracted from the
